// Camera.C

#include "Camera.H"

#include "Carpet.H"
#include "Light.H"


namespace CS148 {


class FogPlane : public DisplayBuilder
{
public:
  FogPlane ( ) : DisplayBuilder () {}
  ~FogPlane () {}

  void buildDisplayList () {
    glMaterialfv ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE,
		   Color(0, .75, 1.0) ); // Deep sky blue

    glRotatef ( 90, 0, 1, 0 );
    glTranslatef ( 0, 0, 100 );
    glutSolidCone ( 500, 50, 8, 1 );
  }

};




Camera::Camera ( Carpet* carpet, bool slowMode )
  : Viewport(0,0,1,1), slowMode(slowMode), 
    carpet(carpet),
    fogPlane( slowMode? 0 : FogPlane().buildNew(true))
{
  setReference ( new PosObj() );
}


Camera::~Camera ()
{
  delete getReference();
}


void Camera::followCarpet ()
{
  Real rot = carpet->getRotation();
  Vec3 cpos = carpet->getPos();
  Real speed = len(carpet->getVelo());

  // Matrix to transform to carpet's local coords
  Mat4 locMat = ( HTrans4( cpos ) *
		  HRot4( Vec3(0,1,0), deg2rad(-rot) ) );

  // Look at a point out in front of the carpet
  PosObj* ref = getReference();
  Vec3 rrelPos = Vec3 ( 2 + speed/100.0, 0, 0 );
  Vec3 refPos = xform ( locMat, rrelPos );

  // Camera trails behind carpet
   Vec3 erelPos = Vec3( -.6 + speed/80.0, .7 - speed/150.0, 0 );
  Vec3 eyePos = xform ( locMat, erelPos );

  bool newLook = ( eyePos != getPos() ||
		   refPos != ref->getPos() );
  if ( newLook ) {
    setPos ( eyePos );
    ref->setPos ( refPos );
//     cerr << "Look at " << refPos << " from " << eyePos 
// 	 << " rot=" << rot << "\n";
    moved();
  }
}





void Camera::render ()
{
  Viewport::render();

  if ( !slowMode ) {
    glPushMatrix();
    carpet->translate();
    carpet->rotate();
    //   fogPlane->call();
    FogPlane().buildDisplayList();
    glPopMatrix();
  }
}




} // namespace CS148


