// Carpet.C

#include "Carpet.H"
#include "Texture.H"

namespace CS148 {


/**********************************************************************
 * CarpetBuilder
 **********************************************************************/

#include "carpet2.c" // Carpet texture image
static Texture* carpTex = 0;



class CarpetBuilder : public DisplayBuilder
{
public:
  CarpetBuilder ()
    : DisplayBuilder() 
  {
    if ( !carpTex ) {
      carpTex = new Texture;
      carpTex->buildMipmaps ( carpet2_img.width,
			      carpet2_img.height,
			      carpet2_img.pixel_data,
			      GL_RGBA );
      carpTex->unbind();
    }
  }

  ~CarpetBuilder ()
  {}

  void set ( int i, int m ) { this->i = i; this->m = m; }

  void buildDisplayList() {
    static const Vec4 carpet ( .8, 1, 1, 0 );
    glShadeModel ( GL_SMOOTH );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, carpet );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_SPECULAR, carpet );
    glMaterialf ( GL_FRONT_AND_BACK, GL_SHININESS, 80 );
    glPolygonMode ( GL_FRONT_AND_BACK, GL_FILL );

     carpTex->bind();

    // Use 2-dim mesh evaluator to produce carpet that waves
    // in the air as it moves
    static const Real amp = .04;
    Real meshpoints[4][4][3];
    for ( int v=0; v < 4; ++v ) {
      for ( int u=0; u < 4; ++u ) {
	meshpoints[v][u][0] = 0.7 * u/4;
	meshpoints[v][u][1] = ( amp * sin ( 3.1416 * ( (u+i)%m )/m ) );
	meshpoints[v][u][2] = (-.375 + .25*v +
			       .25*amp * cos ( 3.1416 * ( (u+i)%m )/m ) );
      }
    }

    glMap2f ( GL_MAP2_VERTEX_3,
	      0.0, 1.0, 3, 4,
	      0.0, 1.0, 12, 4,
	      &meshpoints[0][0][0] );
    glEnable ( GL_MAP2_VERTEX_3 );

    static Real texpoints[2][2][2] = {{{0, 0},{0, 1}},{{1, 0},{1, 1}}};
    glMap2f ( GL_MAP2_TEXTURE_COORD_2,
	      1.0, 0.0, 2, 2,
	      0.0, 1.0, 4, 2,
	      &texpoints[0][0][0] );
    glEnable ( GL_MAP2_TEXTURE_COORD_2 );
    
    glEnable ( GL_AUTO_NORMAL );
    glMapGrid2f ( 4, 0.0, 1.0, 4, 0.0, 1.0 );
    glEvalMesh2 ( GL_FILL, 0, 4, 0, 4 );
    glDisable ( GL_MAP2_VERTEX_3 );
    glDisable ( GL_MAP2_TEXTURE_COORD_2 );
    glDisable ( GL_AUTO_NORMAL );

    carpTex->unbind();
  }

private:

  int i, m;
};





/**********************************************************************
 * Carpet
 **********************************************************************/


Carpet::Carpet ( Ground* ground, bool slowMode )
  : Mover(),
    slowMode(slowMode),
    goingLeft(false), goingRight(false),
    goingFaster(false), goingSlower(false)
{
  setSpeedLimit ( 30, .1 );
  setElevation ( ground, 5, .1 );
  setFrameSpeed ( slowMode? 3 : 6 );
}


Carpet::~Carpet ()
{

}


void Carpet::buildFrames ()
{
  CarpetBuilder cb;

  int m = 10;
  for ( int i = 0; i < m; i++ ) {
    cb.set ( i, m );
    addFrame ( cb.buildNew( true ) );
  }
}


void Carpet::render ( Viewport* vp )
{
  // Assume always visible:
  //if ( !vp->canSee ( this ) ) return;

  glPushMatrix();
  translate();
  rotate();
  renderCurrFrame();
  glPopMatrix();
}


void Carpet::key ( const KeyEvent& ev )
{
  switch ( ev.c ) {

  case 'a':
  case 'A':
    if ( ev.down ) turnLeft();
    goingLeft = ev.down;
    break;

  case 'd':
  case 'D':
    if ( ev.down ) turnRight();
    goingRight = ev.down;
    break;

  case 'w':
  case 'W':
    if ( ev.down ) accelerate();
    goingFaster = ev.down;
    break;

  case 's':
  case 'S':
    if ( ev.down ) decelerate();
    goingSlower = ev.down;
    break;

  default: break;
  }

}


void Carpet::move ( Real interval )
{
  if ( goingLeft ) turnLeft();
  if ( goingRight ) turnRight();
  if ( goingFaster ) accelerate();
  if ( goingSlower ) decelerate();

  // Now do the actual moving
  Mover::move ( interval );
}


void Carpet::turnLeft ()
{
  turn ( 2.5, 15 );
}

void Carpet::turnRight ()
{
  turn ( -2.5, 15 );
}

void Carpet::accelerate ()
{
  Mover::accelerate ( 1 );

  Real fsp = getFrameSpeed();
  fsp *= 1.05;
  if ( fsp > 15 ) fsp = 15;
  setFrameSpeed ( fsp );
}


void Carpet::decelerate ()
{
  Mover::accelerate ( -1 );

  Real fsp = getFrameSpeed();
  fsp /= 1.05;
  if ( fsp < 5 ) fsp = 5;
  setFrameSpeed ( fsp );
}



} // namespace CS148


