// CocoBomb.C

#include "CocoBomb.H"

#include "AliBaba.H"
#include "Ground.H"

namespace CS148 {

class CocoBuilder : public DisplayBuilder
{
public:
  CocoBuilder ()
    : DisplayBuilder()
  {
      
  }
  
  ~CocoBuilder () {}

  void buildDisplayList () {
    static const Vec4 coco ( .4, .3, .25, 1.0 );
    glShadeModel ( GL_SMOOTH );
    glMaterialfv ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, coco ); 
    glMaterialfv ( GL_FRONT, GL_SPECULAR, coco );
    glMaterialf ( GL_FRONT, GL_SHININESS, 0 );
    glutSolidSphere ( .2, 12, 8 );
  }

};

DisplayList* CocoBomb::dlist = 0;
 

CocoBomb::CocoBomb ( AliBaba* app, Ground* ground )
  : Mover(),
    app(app), ground(ground)
{
  if ( !dlist ) dlist = CocoBuilder().buildNew(true);
  setBoundRadius ( .4 );
}


CocoBomb::~CocoBomb ()
{

}


void CocoBomb::render ( Viewport* vp )
{
  if ( !vp->canSee( this ) ) return;

  glPushMatrix();
  translate();
/*   rotate(); */
  dlist->call();
  glPopMatrix();
}


void CocoBomb::move ( Real interval )
{
//   cerr << "Bomb::move " << interval << " " << this 
//        << "  pos=" << getPos() << " velo=" << getVelo() << "\n";

  // Acceleration due to gravity
  static const Real G = 10; // 10 m/s^2
  Vec3 velo = getVelo();
  velo[1] -= G*interval;
  setVelo ( velo );

  // Update position
  Mover::move ( interval );

  // Check for explosion
  if ( hitGround() ) {
    explode();
  }

}


bool CocoBomb::hitGround ()
{
  const Vec3& p = getPos();
  Real y = ground->getHeight ( p[0], p[2] );
  return ( p[1] <= y );
}


void CocoBomb::explode ()
{
  app->exploded( this );
}



} // namespace CS148


