// Explosion.C

#include "Explosion.H"

#include "AliBaba.H"
#include "Light.H"

namespace CS148 {


class ExplosionBuilder : public DisplayBuilder
{
public:
  static void build ( Explosion* expl ) {
    if ( frames.size() == 0 ) render();
    for ( vector<DisplayList*>::iterator iter = frames.begin();
	  iter != frames.end(); ++iter ) {
      expl->addFrame ( *iter );
    }
  }

  static vector<DisplayList*> frames;


  static void render () {
    for ( GLfloat scale = 1.0; scale <= 5.0; scale *= 1.7 ) {
      frames.push_back ( ExplosionBuilder(scale).buildNew(true) );
    }
  }


  ExplosionBuilder ( GLfloat scale )
    : DisplayBuilder(), scale(scale)
  {}

  ~ExplosionBuilder () {}

  GLfloat scale;

  void buildDisplayList () {
    glScalef ( scale, scale, scale );
    //glTranslatef ( 0, .5, 0 );

    glPushAttrib ( GL_COLOR_BUFFER_BIT | GL_ENABLE_BIT | GL_EVAL_BIT );
    glShadeModel ( GL_SMOOTH );
    glEnable ( GL_AUTO_NORMAL );

    glEnable ( GL_BLEND );
    glBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

    glMaterialfv ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE,
		   Color ( .824, .706, .549, .6 ) );
    glMaterialfv ( GL_FRONT, GL_SPECULAR, Color ( .824, .706, .549, .6 ) );
    glMaterialf ( GL_FRONT, GL_SHININESS, 100 );

    glutSolidSphere ( 1.0, 10, 10 );

    glPopAttrib();

    Light expl ( 4, Vec4(0,0,0,1) );
    expl.setDiffuse ( Vec3( .8, 0, 0 ) );
    expl.setAmbient ( Vec3( .8, .6, 0 ) );
    expl.setSpecular ( Vec3( 1, 1, 0 ) );
    expl.setAttenuation ( .1, 0, .005 + .01*(scale - 1.0) );
    expl.draw();
  }

};


vector<DisplayList*> ExplosionBuilder::frames;




Explosion::Explosion ( AliBaba* app, const Vec3& p )
  : PosObj(), Animated(),
    app(app)
{
  setPos ( p );
  setFrameSpeed ( 12 );
  buildFrames();
  setBoundRadius ( 5 );
  app->addAnim ( this );
  app->addPosObj ( this );
}


Explosion::~Explosion ()
{
  glDisable ( GL_LIGHT4 );
}


void Explosion::buildFrames ()
{
  ExplosionBuilder::build ( this );
}


void Explosion::render ( Viewport* vp )
{
  if ( !vp->canSee ( this ) ) return;

  glPushMatrix();
  translate();
  renderCurrFrame();
  glPopMatrix();
}




void Explosion::endAnimSequence ()
{
  app->removeAnim ( this );
  delete this;
}



} // namespace CS148


