// Ground.H

#ifndef _Ground_H_
#define _Ground_H_

#include "GlutApp.H"

namespace CS148 {

class Ground;
class Texture;

class GroundPatch : public PosObj,
		    protected DisplayBuilder
{
public:
  GroundPatch ( Ground* ground, int r1, int c1, int r2, int c2 );
  ~GroundPatch ();

  void redraw ( int r1, int c1, int r2, int c2 );
  void render ( Viewport* vp );

  static Texture* wallTexture () { return wallTex; }

private:

  void divide ();
  void buildDisplayList();
  void setGndMaterial ( GLfloat y, GLfloat maxy, int r, int c );
  void setWallMaterial ( GLfloat y );

  bool _render ( Viewport* vp );

  Ground* ground;
  bool didDivision;
  int r1, c1, r2, c2;
  GroundPatch* patch[4];
  DisplayList* dlist;
  static Texture* wallTex;
};
  


class Ground
{
public:
  Ground ( GLfloat w, GLfloat h, GLfloat step, bool slowMode );
  ~Ground ();

  GLfloat getWidth () const { return w; }
  GLfloat getLength () const { return h; }
  void getSize ( GLfloat& w, GLfloat& h ) { w = this->w; h = this->h; }

  GLfloat getHeight ( GLfloat x, GLfloat z );
  GLfloat getAvgHeight ( GLfloat x, GLfloat z, GLfloat rad );
  GLfloat getMinHeight ( GLfloat x, GLfloat z, GLfloat rad );

  GroundPatch* getTopPatch();

  void explosion ( GLfloat x, GLfloat z );

private:

  void makeTerrain ();
  void buildWalls();
  void buildDunes();

  void rebuildPatch ( int r1, int c1, int r2, int c2 );

  void mapPos ( GLfloat x, GLfloat z, int& r, int& c );
  void unmapPos ( int r, int c, GLfloat& x, GLfloat& z );
  GLfloat mapAt ( GLfloat x, GLfloat z );
  int mapIndex ( int r, int c );
  void set ( int r, int c, GLfloat y );
  void incr ( int r, int c, GLfloat dy );
  bool isWall ( int r, int c ) const;

  bool slowMode;
  GLfloat w, h;
  int rows, cols;
  GLfloat dx, dz;
  GLfloat* map;
  GLfloat maxy;

  friend class GroundPatch;
  GroundPatch* topPatch;

  bool* expMap;
};



} // namespace CS148


#endif /* _Ground_H_ */

