// Light.H

#ifndef _Light_H_
#define _Light_H_

#include "GlutApp.H"

namespace CS148 {

static const GLenum lightEnum[8] = { GL_LIGHT0, GL_LIGHT1,
				     GL_LIGHT2, GL_LIGHT3,
				     GL_LIGHT4, GL_LIGHT5,
				     GL_LIGHT6, GL_LIGHT7 };
  

class Light : public DisplayBuilder
{
public:
  Light ( unsigned short num, const Vec4& pos )
    : DisplayBuilder(), num(num), pos(pos),
      ambient(), diffuse(), specular(),
      haveAmbient(false), haveDiffuse(false), haveSpecular(false),
      spotAngle(0), spotExp(0), spotDir(), haveSpot(false),
      attConst(0), attLin(0), attQuad(0), haveAtten(false)
  {
    setNum ( num );
  }

  ~Light () {}

  void setNum ( unsigned short num ) {
    assert ( num < 8 );
    glEnable ( GL_LIGHTING );
    glEnable ( lightEnum[num] );
  }

  void setPos ( const Vec4& v ) { pos = v; }

  void setAmbient ( const Vec3& v ) { ambient = v; haveAmbient = true; }
  void setDiffuse ( const Vec3& v ) { diffuse = v; haveDiffuse = true; }
  void setSpecular ( const Vec3& v ) { specular = v; haveSpecular = true; }

  void setSpot ( GLfloat cut, const Vec3& d, GLfloat exp = 0 ) {
    spotAngle = cut;
    spotDir = d;
    spotExp = exp;
    haveSpot = true;
  }

  void setAttenuation ( GLfloat c, GLfloat l = 0, GLfloat q = 0 ) {
    attConst = c;
    attLin = l;
    attQuad = q;
    haveAtten = true;
  }

  void draw () { buildDisplayList(); }

  static void setGlobalAmbient ( const Vec3& v ) {
    glEnable ( GL_LIGHTING );
    glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, v );
  }

  void enable () { glEnable ( lightEnum[num] ); }
  void disable () {  glDisable ( lightEnum[num] ); }
    
private:

  void buildDisplayList () {
    glLightfv ( lightEnum[num], GL_POSITION, pos );

    if ( haveAmbient ) glLightfv ( lightEnum[num], GL_AMBIENT, ambient );
    if ( haveDiffuse ) glLightfv ( lightEnum[num], GL_DIFFUSE, diffuse );
    if ( haveSpecular ) glLightfv ( lightEnum[num], GL_SPECULAR, specular );

    if ( haveSpot ) {
      glLightf ( lightEnum[num], GL_SPOT_CUTOFF, spotAngle );
      glLightfv ( lightEnum[num], GL_SPOT_DIRECTION, spotDir );
      glLightf ( lightEnum[num], GL_SPOT_EXPONENT, spotExp );
    }

    if ( haveAtten ) {
      glLightf ( lightEnum[num], GL_CONSTANT_ATTENUATION, attConst );
      glLightf ( lightEnum[num], GL_LINEAR_ATTENUATION, attLin );
      glLightf ( lightEnum[num], GL_QUADRATIC_ATTENUATION, attQuad );
    }

//     // Cube to show light position
//     glTranslatef ( pos[0], pos[1], pos[2] );
//     glDisable(GL_LIGHTING);
//     glColor3f(0.0, 1.0, 1.0);
//     glutWireCube(0.1);
//     glEnable(GL_LIGHTING);
    
  }

  unsigned short num;
  Vec4 pos;
  Vec3 ambient, diffuse, specular;
  bool haveAmbient, haveDiffuse, haveSpecular;

  GLfloat spotAngle, spotExp;
  Vec3 spotDir;
  bool haveSpot;

  GLfloat attConst, attLin, attQuad;
  bool haveAtten;

};



} // namespace CS148


#endif /* _Light_H_ */

