// Mover.H

#ifndef _Mover_H_
#define _Mover_H_

#include "GlutApp.H"

namespace CS148 {

class Ground;
class Turn;

class Mover : public PosObj
{
public:
  Mover ();
  ~Mover ();

  virtual void move ( Real interval );
  Vec3 getPosAfter ( Real interval );

  void setVelo ( const Vec3& v );
  const Vec3& getVelo () const { return velo; }
  Real getRotation () const { return rotation; }

  void setSpeedLimit ( Real maxsp, Real minsp = 0 ) 
  { maxSpeed = maxsp; minSpeed = minsp; }

  void setElevation ( Ground* g, Real el, Real dr = 1.0 );

  void setBoundary ( Real x1, Real z1, Real x2, Real z2 );

  void rotate () const { glRotatef ( -getRotation(), 0, 1, 0 ); }
    

  void addTurn ( Turn* t );
  void turn ( Real deg, Real rate );
  void accelerate ( Real sp );

  void warp ( Real x, Real z );

private:

  void adjustHeight ( Real interval );
  void applyTurns ( Real interval );
  void applyVelo ( Real interval );
  void checkBoundary ();


  Vec3 velo;
  Real rotation;
  Real maxSpeed, minSpeed;

  Ground* ground;
  Real targetHeight, descentRate;

  static const unsigned int maxTurns = 5;
  vector<Turn*> turns;

  bool bounded;
  Real x1, z1, x2, z2;
};



// Encapsulate a direction change executed over time
// This is a rotation of a total number of degrees, with
// a degrees-per-second rate.
// A Mover can have multiple simultaneous turns in effect,
// mostly to give the effect of inertia when quickly
// alternating directions
class Turn
{
public:
  Turn ( Real total, Real rate ) : left(total), rate(rate) {}

  Vec3 apply ( const Vec3& v, Real interval ) {
    Real deg = rate * interval;
    if ( fabs(deg) > fabs(left) ) deg = left;
    left -= deg;

    return xform ( Rot3(Vec3(0,1,0), deg2rad(deg)), v );
  }

  bool completed () const {
    return (rate > 0)? (left <= 0.0) : (left >= 0.0);
  }

private:

  Real left;
  Real rate;

};





} // namespace CS148


#endif /* _Mover_H_ */

