// Particles.C

#include "Particles.H"

#include "AliBaba.H"
#include "Mover.H"

namespace CS148 {

static const GLfloat colors[10][4] = { 
  { 1.00, 0.00, 0.00, 0.34 },
  { 1.00, 0.00, 0.00, 0.10 },
  { 0.35, 0.00, 0.00, 0.10 },

  { 0.74, 0.72, 0.42, 0.50 },
  { 0.78, 0.71, 0.47, 0.20 },
  { 0.96, 0.64, 0.38, 0.80 },

  { 0.87, 0.72, 0.53, 0.98 },
  { 0.70, 0.13, 0.13, 0.40 },
  { 1.00, 0.39, 0.28, 0.60 },
  { 1.00, 0.27, 0.00, 0.10 }

};


class ParticleBuilder : public DisplayBuilder
{
public:
  static vector<DisplayList*> sizes;

  static void render () {
    for ( GLfloat scale = 1.0; scale <= 15.0; scale += .25 ) {
      sizes.push_back ( ParticleBuilder(scale).buildNew(true) );
    }
  }


  ParticleBuilder ( GLfloat scale )
    : DisplayBuilder(), scale(scale)
  {}

  ~ParticleBuilder () {}

  GLfloat scale;

  void buildDisplayList () {
    glColor4fv ( colors[rand()%10] );
    glPointSize ( scale );
    glBegin ( GL_POINTS );
    glVertex3f ( 0, 0, 0 );
    glEnd();
  }

};


vector<DisplayList*> ParticleBuilder::sizes;


static unsigned int numParts = 0;
  static const unsigned int maxParts = 300;



class Particle : public Mover
{
public:
  Particle ( DisplayList* dlist, const Vec3& p, const Vec3& v )
    : Mover(), dlist(dlist)
  {
    setPos(p);
    setVelo(v);
  }

  ~Particle () {}

  void move ( Real interval ) {
    // Acceleration due to gravity
    static const Real G = 10; // 10 m/s^2
    Vec3 velo = getVelo();
    velo[1] -= G*interval;
    setVelo ( velo );
    
    // Update position
    Mover::move ( interval );
  }
  
  void render () {
    glPushMatrix();
    translate();
    dlist->call();
    glPopMatrix();
  }
  
private:
  
  DisplayList* dlist;
};






Particles::Particles ( AliBaba* app, const Vec3& p, const Vec3& v )
  : Mover(), Functor(), app(app), parts(), timer()
{
  setPos ( p );
  setBoundRadius ( 200 );
  app->addPosObj ( this );
  app->addMover ( this );

  if ( ParticleBuilder::sizes.size() == 0 ) ParticleBuilder::render();

  unsigned int n = 30 + rand()%40;
  while ( n-- && numParts < maxParts ) {
    Vec3 velo = v + sphere2cart( Vec3 ( 20 + rand()%40,
					rand() % 60,
					rand() % 360 ) );

    unsigned int sz = rand() % ParticleBuilder::sizes.size();

    parts.push_back ( new Particle
		      ( ParticleBuilder::sizes[sz], p, velo ) );
    numParts++;
  }
}


Particles::~Particles ()
{
  numParts -= parts.size();
  for ( vector<Particle*>::iterator iter = parts.begin();
	iter != parts.end(); ++iter )
    delete *iter;
}


void Particles::func ()
{
  app->removeMover ( this );
  app->removePosObj ( this );
  delete this;
}


void Particles::move ( Real interval )
{
  for ( vector<Particle*>::iterator iter = parts.begin();
	iter != parts.end(); ++iter )
    (*iter)->move ( interval );

  if ( timer.getElapsed() > 2.0 ) {
    app->after ( 1, this );
  }
}


void Particles::render ( Viewport* vp )
{
  if ( !vp->canSee ( this ) ) return;

  glDisable ( GL_LIGHTING );
  glEnable ( GL_POINT_SMOOTH ); // Round points
  glEnable ( GL_BLEND );
  glBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

  for ( vector<Particle*>::iterator iter = parts.begin();
	iter != parts.end(); ++iter )
    (*iter)->render();

  glDisable ( GL_POINT_SMOOTH );
  glDisable ( GL_BLEND );
  glEnable ( GL_LIGHTING );

}




} // namespace CS148


