// PauseView.H

#ifndef _PauseView_H_
#define _PauseView_H_

#include "GlutApp.H"


namespace CS148 {


class PauseView : public Viewport
{
public:
  PauseView ( AliBaba* app, int x, int y, int w, int h )
    : Viewport( x, y, w, h ), app(app)
  {

  }

  ~PauseView () {}

  enum Name { nameNothing = 0,
	      nameRestart,
	      nameQuit,
  };

  void reshaped ( int x, int y, int w, int h ) {
    Viewport::reshaped ( x, y, w, h );
    setOrtho ( x, x+w, y, y+h, -2, 2 );
  }
    
  void render () {
    glPushAttrib ( GL_ENABLE_BIT | GL_EVAL_BIT | GL_COLOR_BUFFER_BIT );
    glDisable(GL_LIGHTING);

    glEnable ( GL_BLEND );
    glBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

    glPushMatrix();
    glLoadIdentity();

    // Render and do nameing altogether
    glInitNames();
    glPushName(0);

    // Menus
    startMenus( 2 );
    drawMenu ( nameRestart, "Restart" );
    drawMenu ( nameQuit, "Quit" );

    glPopName();

    // Background
    int x, y, w, h; getArea ( x, y, w, h );
    glColor4fv ( Color ( 0, 0, 0, .3 ) );
    glPolygonMode ( GL_FRONT_AND_BACK, GL_FILL );
    glBegin ( GL_QUADS );
    glVertex3f ( 0, 0, -.25 );
    glVertex3f ( w, 0, -.25 );
    glVertex3f ( w, h, -.25 );
    glVertex3f ( 0, h, -.25 );
    glEnd();

    glPopMatrix();
    glPopAttrib();
  }

  void startMenus ( int n ) {
    currMenu = 0;
    maxMenus = n;

    mwidth = app->getScreenWidth() - 400;
    if ( mwidth < 150 ) mwidth = 150;
    mheight = 50;
    dheight = 15;

    startx = (app->getScreenWidth() - mwidth) / 2;
    starty = (app->getScreenHeight() - maxMenus*mheight -
	      maxMenus* ( dheight - 1 ) );
  }

  void drawMenu ( Name name, const char* str ) {
    glLoadName ( name );

    glPushMatrix();
    glTranslatef ( startx,
		   starty - (currMenu+1) * ( mheight + dheight ),
		   0.25 );
    glColor4fv ( Color ( 0, 0, 1, .2 ) );
    glPolygonMode ( GL_FRONT_AND_BACK, GL_FILL );
    glBegin ( GL_QUADS );
    glVertex3f ( 0, 0, -.3 );
    glVertex3f ( mwidth, 0, -.3 );
    glVertex3f ( mwidth, mheight, -.3 );
    glVertex3f ( 0, mheight, -.3 );
    glEnd();

    glColor4fv ( Color ( 1, 0, 0, 1 ) );
    glLineWidth ( 3.0 );
    glPolygonMode ( GL_FRONT_AND_BACK, GL_LINE );
    glBegin ( GL_QUADS );
    glVertex2f ( 0, 0 );
    glVertex2f ( mwidth, 0 );
    glVertex2f ( mwidth, mheight );
    glVertex2f ( 0, mheight );
    glEnd();

    glColor4fv ( Color ( 0, 1, 0, 1 ) );
    glTranslatef ( 0, 0, .5 );

    Real strWidth = 100; // Should calculate...
    renderBitmapString ( (mwidth - strWidth) / 2, 16,
			 str, GLUT_BITMAP_HELVETICA_18  );

    glPopMatrix();
    currMenu++;
  }


  void motion ( const MotionEvent& ev ) {
    setPos ( Vec3( ev.x, ev.y, 0 ) );
  }

  void pick ( const ButtonEvent& ev ) {
    // Prepare for selection / picking
    static const int BUFSIZE = 512;
    GLuint selbuf[BUFSIZE];
    glSelectBuffer ( BUFSIZE, selbuf );


    // Set pick projection matrix
    // Small region around the mouse
    glMatrixMode ( GL_PROJECTION );
    glPushMatrix();
    glLoadIdentity();

    GLint vp[4];
    glGetIntegerv ( GL_VIEWPORT, vp );
    gluPickMatrix ( ev.x-1, ev.y-1, 3, 3, vp );
    setProjection (); // Does glOrtho()

    // Render in selection mode
    glMatrixMode ( GL_MODELVIEW );
    (void) glRenderMode ( GL_SELECT );
    render();
    
    // Restore projection matrix
    glMatrixMode ( GL_PROJECTION );
    glPopMatrix();
    glMatrixMode ( GL_MODELVIEW );

    // Get hit records
    glFlush();
    GLint hits = glRenderMode ( GL_RENDER );

    app->flush();
    processPicked ( hits, selbuf );
  }

  void processPicked ( GLint hits, GLuint selbuf[] ) {
//     cerr << "hits = " << hits << "\n";

    GLuint* ptr = selbuf;
    for ( int i = 0; i < hits; ++i ) {
      GLuint names = *ptr++;  // num names for this hit
      ptr++;//GLfloat z1 = (*ptr++) / 0x7fffffff;
      ptr++;//GLfloat z2 = (*ptr++) / 0x7fffffff;
//       cerr << "hit " << i << ": names=" << names
// 	   << " z1=" << z1 << " z2=" << z2 << "\n";
      for ( GLuint j = 0; j < names; ++j ) {
	GLuint name = *ptr++;
// 	cerr << "  name " << j << " = " << name << "\n";

	switch ( name ) {
	case nameNothing:
	  break;

	case nameRestart:
	  cerr << "Restart: not implemented\n";
	  break;

	case nameQuit:
	  cerr << "Bye!\n";
	  exit ( 0 );
	  break;

	default:
	  cerr << "Unknown selection\n";
	  break;
	}
      } // for ( j... )
    } // for ( i... )
  }
      
private:

  AliBaba* app;

  uint currMenu, maxMenus;
  Real mwidth, mheight, dheight;
  Real startx, starty;

  
};


} // namespace CS148

#endif /* _PauseView_H_ */
