// Rider.C

#include "Rider.H"

#include "AliBaba.H"
#include "Ground.H"
#include "Cave.H"
#include "Texture.H"

#include "Rider.ac.H"

namespace CS148 {


#include "horse1.c" // Horse texture image
static Texture* bodyTex = 0;

#include "horsehead1.c" // Horse head texture image
static Texture* headTex = 0;


class Horse : public DisplayBuilder
{
public:
  Horse () : DisplayBuilder() {}
  ~Horse () {}

  void buildDisplayList () {
    glAlphaFunc ( GL_NOTEQUAL, 0.0 );  
    glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );

    bodyTex->bind();
    HorseBody().immediate();
    bodyTex->unbind();

//     headTex->bind();
//     HorseHead().immediate();
//     headTex->unbind();

    glAlphaFunc ( GL_ALWAYS, 1 );
  }
};
  
  
    

Rider::Rider ( AliBaba* app, Ground* ground, Cave* cave )
  : Mover(), Animated(),
    app(app), ground(ground), cave(cave),
    goal(), nearGoal(false)
{
  setSpeedLimit ( 25, 10 );
  setElevation ( ground, .1, .8 );
  setFrameSpeed ( 5 );

  // Heading to the cave at the far end
  goal = cave->getPos();
  goal[1] = ground->getHeight ( goal[0], goal[2] );

  // Random start time for planning, so that
  // all are staggered
  app->after ( rand() % 2000, this );
}


Rider::~Rider ()
{
  app->cancelAfter ( this );

}


void Rider::buildFrames ()
{
  if ( !bodyTex ) {
    bodyTex = new Texture;
    bodyTex->buildMipmaps ( horse1_img.width,
			     horse1_img.height,
			     horse1_img.pixel_data,
			     GL_RGBA );
    bodyTex->unbind();
  }

  if ( !headTex ) {
    headTex = new Texture;
    headTex->buildMipmaps ( head1_img.width,
			     head1_img.height,
			     head1_img.pixel_data,
			     GL_RGBA );
    headTex->unbind();
  }

  glPushMatrix();
  glLoadIdentity();

  addFrame ( Horse().buildNew() );

  glPopMatrix();
}


void Rider::render ( Viewport* vp )
{
  if ( !vp->canSee( this ) ) return;

  glPushMatrix();
  translate();
  rotate();
  renderCurrFrame();
  glPopMatrix();
}


void Rider::move ( Real interval )
{
  Mover::move ( interval );

  // Slow down if going up hill, speed up down hill

}


void Rider::nearMiss ()
{
  // Turn away in a random direction
  int turnDeg = 30 + rand()%40;
  if ( turnDeg % 2 ) turnDeg *= -1;

  turn ( turnDeg, 60 );
  accelerate ( 2 );
}


void Rider::hitByBomb ()
{
  // Ouch!
  app->killed ( this );
}


void Rider::func ()
{
  if ( plan() ) {
    static const unsigned int planInterval = 500;
    app->after ( planInterval, this );
  }
}


bool Rider::plan ()
{
  // Turn to the goal --
  // What direction to go?
  // Make a turn to get there
  Vec3 toGoal = goal - getPos();
//   cerr << "toGoal = " << toGoal << "\n";
  Vec3 toGoalsp = cart2sphere ( toGoal );

  Real deltaRot = getRotation() - toGoalsp[2];

  // Turn more slowly the farther from the goal
  Real deltaRate = 30*(1.0 - len(toGoal) / ground->getLength() );
//   cerr << "delta rot = " << deltaRot
//        << ", rate = " << deltaRate << "\n";
  if ( !nearGoal && fabs(deltaRot) > deltaRate ) 
    deltaRot = deltaRate * ( deltaRot/deltaRot );

  turn ( deltaRot, deltaRate );

  // Slow down a bit over time, unless near the end
  accelerate ( nearGoal? .2 : -.1 );

  // Open the door if getting close
  Real dist = len(toGoal);
  if ( dist < 12 ) {
    app->reachedCave ( this );
    return false;
  }
  else if ( dist < 100 ) {
    nearGoal = true;
    cave->openSesame();
  }

  return true;
}



} // namespace CS148


