// Texture.H

#ifndef _Texture_H_
#define _Texture_H_

#include "GlutApp.H"

namespace CS148 {


class Texture
{
public:
  Texture ( bool twoD = true );
  virtual ~Texture ();

  void bind ();
  void unbind ();

  virtual void init ();

  void buildMipmaps ( GLsizei w, GLsizei h, const void* data,
		      GLenum format = GL_RGB,
		      GLenum type = GL_UNSIGNED_BYTE ) {
    bind();
    gluBuild2DMipmaps ( GL_TEXTURE_2D, 3, w, h, format, type, data );
  }


protected:

  void setModulatedMipmap () {
    assert ( texDim == GL_TEXTURE_2D );

    bind();

    glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

    // when texture area is small, bilinear filter the closest mipmap
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
		     GL_LINEAR_MIPMAP_NEAREST );

    // when texture area is large, bilinear filter the original
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

    // the texture wraps over at the edges (repeat)
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
  }


private:

  GLuint texId;
  GLenum texDim;

  static const GLuint NO_TEX = 0;//0xffffffff;
  static GLuint boundTex;

};



} // namespace CS148


#endif /* _Texture_H_ */

