
#include "GlutApp.H"
#include "Light.H"

#include <iostream>
#include <vector>

using namespace std;
using namespace CS148;

#include "Rider.ac.C"


class Axes : public DisplayBuilder
{
public:
  Axes () : DisplayBuilder() {}
  ~Axes () {}

  void buildDisplayList () {
    glDisable ( GL_LIGHTING );
    
    glBegin ( GL_LINES );
    glColor3fv ( Color::BLUE() );
    glVertex3f ( 0, 0, 0 ); glVertex3f ( 10, 0, 0 );

    glColor3fv ( Color::GREEN() );
    glVertex3f ( 0, 0, 0 ); glVertex3f ( 0, 10, 0 );

    glColor3fv ( Color::RED() );
    glVertex3f ( 0, 0, 0 ); glVertex3f ( 0, 0, 10 );
    glEnd ();

    glEnable ( GL_LIGHTING );
  }
};


class LitTube : public DisplayBuilder
{
public:
  LitTube () : DisplayBuilder() {}
  ~LitTube () {}

  void buildDisplayList () {
    glPushAttrib ( GL_COLOR_BUFFER_BIT );
    glEnable ( GL_BLEND );
    glBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

    Vec4 glow ( 1, 1, 1, .5 );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, glow );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_SPECULAR, glow );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_EMISSION, glow );

    glutSolidSphere ( 1, 10, 10 );


    Vec3 mat ( .5, .5, .5 );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_SPECULAR, mat );
    glMaterialfv ( GL_FRONT_AND_BACK, GL_EMISSION, mat );
    glBegin ( GL_QUADS );
    glNormal3f ( 0, 0, 1 );
    glVertex3f ( 5, -5, -3 );
    glVertex3f ( 5, 5, -3 );
    glVertex3f ( -5, 5, -3 );
    glVertex3f ( -5, -5, -3 );

    glNormal3f ( 0, 0, 1 );
    glVertex3f ( 10, -10, -5 );
    glVertex3f ( 10, 10, -5 );
    glVertex3f ( -10, 10, -5 );
    glVertex3f ( -10, -10, -5 );
    glEnd();

  }
};



class Viewer : public GlutApp
{
public:
  static GlutApp* factory () { return new Viewer; }

  Viewer ()
    : GlutApp()
  {

  }

  ~Viewer () {

  }


  Viewport* camera;

  void init () {
    GlutApp::init();
    //glEnable ( GL_CULL_FACE );
    //    glEnable ( GL_COLOR_MATERIAL );
    glLightModeli ( GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE );
    glLightModeli ( GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE );
    glShadeModel ( GL_SMOOTH );
    glEnable ( GL_AUTO_NORMAL );

    setFrameRate ( 18 );

    camera = new Viewport ( 0, 0, getScreenWidth(), getScreenHeight() );
    camera->setPos ( Vec3( 10, 10, 10 ) );//10, 10, 10 ) );
    camera->setReference ( new PosObj() );
    camera->setPerspective ( 60, 0, 0.1, 150.0 );
    addViewport ( camera );

    buildWorld();
  }


  void reshape ( int w, int h ) {
    GlutApp::reshape ( w, h );
    camera->reshaped( 0, 0, w, h );
  }


  void keyboard ( const KeyEvent& ev ) {
    switch ( ev.c ) {
    case 'q':
    case 'Q':
      exit ( 0 );
      break;
    default:
      break;
    }
    
  }
 
 
  void button ( const ButtonEvent& ev ) {

  }


  void motion ( const MotionEvent& ev ) {
    if ( ev.button == Left ) {
      rotate ( ev.dx, ev.dy );
    }
    
    else if ( ev.button == Right ) {
      zoom ( ev.dx, ev.dy );
    }
  }


  static float deg2rad ( float deg ) {
    static const float pi_over_180 = 3.1413 / 180.0;
    return deg * pi_over_180;
  }
  

  void rotate ( int dx, int dy ) {
    // Dragging all the way across the screen would cause a full rotation.
    // Thus:
    //         drot      dx           delev       dy
    //        ------ = -------       ------- = --------
    //         360      width          360      height
    
    if ( abs(dx) > abs(dy) ) {
      float drot = dx * 360.0 / getScreenWidth();
      
      Vec3 pos = cart2sphere( camera->getPos() );
      pos[2] += drot;
      camera->setPos ( sphere2cart ( pos ) );
    }
    
    else {
      float delev = dy * 360.0 / getScreenHeight();

      Vec3 pos = cart2sphere( camera->getPos() );
      pos[1] += delev;
      camera->setPos ( sphere2cart ( pos ) );
    }

    glLoadIdentity();
    camera->moved();
  }


  void zoom ( int dx, int dy ) {
    if ( dy == 0 ) return;

    // Zoom in = drag mouse up
    // Zoom out = drag mouse down
    
    Vec3 rpos = camera->getReference()->getPos() - camera->getPos();

    float dz = len(rpos) * ( 1.0 * dy / getScreenHeight() );
    camera->setPos ( camera->getPos() + rpos * dz );

    glLoadIdentity();
    camera->moved();
  }


  void buildWorld ()
  {
    clearDisplayLists();
    glMatrixMode ( GL_MODELVIEW );
    glPushMatrix();
    glLoadIdentity();

//     clear ( Color ( .8, .7, .6 ) );
    Light::setGlobalAmbient ( Vec3(.5,.5,.5) );
    Light light ( 1, Vec3(0,0,0) );
    light.setDiffuse ( Vec3( 0, 0, 1 ) );
    light.setSpecular ( Vec3( 0, 0, 1 ) );
    light.setAttenuation ( .1 );
    addDisplayList ( light.buildNew(true) );

    Light l2 ( 1, Vec3(0, 5, 5) );
    l2.setDiffuse ( Vec3( 1, 0, 0 ) );
    l2.setSpecular ( Vec3( 1, 0, 0 ) );
    light.setAttenuation ( .4 );
    addDisplayList ( l2.buildNew(true) );


     addDisplayList ( Axes().buildNew() );
//     addDisplayList ( HorseBody().buildNew() );

    addDisplayList ( LitTube().buildNew() );


    glPopMatrix();
  }

  void display ()
  {
    GlutApp::display();

  }

};


int main ( int argc, char** argv )
{
  GlutApp* app = GlutApp::create ( argc, argv, Viewer::factory );
  app->init ( "Viewer", 800, 600, GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB );
  app->run();
  return -1; // Shouldn't get here
}

